﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Collections.ObjectModel;
using IDTechSDK;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace L100_Simple_Demo_UWP
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
            setBaudRateList();
            IDT_L100.setCallback(MessageCallBack);

            //USB Connection:
            Task.Run(() => IDT_L100.useUSB());

            //Serial Connection:
            //Task.Run(() => IDT_L100.useSerialPort(1, 9600, false));	            
        }

        private void setBaudRateList()
        {
            ObservableCollection<string> baudRateList = new ObservableCollection<string>();

            baudRateList.Add("Default");
            baudRateList.Add("2400");
            baudRateList.Add("4800");
            baudRateList.Add("9600");
            baudRateList.Add("19200");
            baudRateList.Add("38400");
            baudRateList.Add("115200");

            comboBoxBaudRate.DataContext = baudRateList;
            comboBoxBaudRate.SelectedIndex = 0;
        }

        private void rbUSB_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() => IDT_L100.useUSB());
        }

        private void rbSerial_Click(object sender, RoutedEventArgs e)
        {
            Int32 portNumber = Convert.ToInt32(tbCOMPort.Text);

            if (comboBoxBaudRate.SelectedIndex == 0)
            {
                Task.Run(() => IDT_L100.useSerialPort(portNumber, false));
            }
            else
            {
                Int32 baudRate = Convert.ToInt32(comboBoxBaudRate.SelectedItem);

                Task.Run(() => IDT_L100.useSerialPort(portNumber, baudRate, false));
            }
        }

        private void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data, IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode)
        {
            switch (state)
            {
                case DeviceState.ToConnect:
                    SetOutputText("To connect\r\n");
                    break;
                case DeviceState.DefaultDeviceTypeChange:
                    //The SDK is changing the default device to IDT_DEVICE_TYPES type
                    SetOutputText("Callback:DefaultDeviceTypeChange\n");
                    connectionStatus.Text = "SmartPIN L100 connected.";
                    connectionStatus.Background = new SolidColorBrush(Windows.UI.Colors.Green);
                    break;
                case DeviceState.Connected:
                    //A connection has been made to IDT_DEVICE_TYPES type
                    SetOutputText("Callback:Connected\n");
                    connectionStatus.Text = "SmartPIN L100 connected.";
                    connectionStatus.Background = new SolidColorBrush(Windows.UI.Colors.Green);
                    break;
                case DeviceState.Disconnected:
                    //A disconnection has occured with IDT_DEVICE_TYPES type
                    SetOutputText("Callback:Disconnected\n");
                    connectionStatus.Text = "SmartPIN L100 not connected.";
                    connectionStatus.Background = new SolidColorBrush(Windows.UI.Colors.Red);
                    break;
                case DeviceState.ConnectionFailed:
                    SetOutputText("Connection Failed\r\n");
                    break;
                case DeviceState.Notification:
                    SetOutputText("Notification\r\n");
                    break;
                case DeviceState.TransactionData:
                    if (cardData == null) break;
                    //output parsed card data
                    SetOutputText("Return Code: " + transactionResultCode.ToString() + "\r\n");
                    if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA)
                    {
                        SetOutputText("PIN Data received:\r\nKSN: " + cardData.pin_KSN + "\r\nPINBLOCK: " + cardData.pin_pinblock + "\r\nKey Entry: " + cardData.pin_KeyEntry + "\r\n");
                        return;
                    }
                    break;
                case DeviceState.DataReceived:
                    SetOutputTextLog(" IN: " + Common.getHexStringFromBytes(data) + "\r\n");
                    break;
                case DeviceState.DataSent:
                    SetOutputTextLog(" OUT: " + Common.getHexStringFromBytes(data) + "\r\n");
                    break;
                case DeviceState.CommandTimeout:
                    SetOutputText("Command Timeout\r\n");
                    break;
                case DeviceState.ToSwipe:
                    SetOutputText("To Swipe\r\n");
                    break;
                case DeviceState.MSRDecodeError:
                    SetOutputText("MSR Decode Error\r\n");
                    break;
                case DeviceState.ToTap:
                    SetOutputText("To Tap\r\n");
                    break;
                case DeviceState.SwipeTimeout:
                    SetOutputText("Swipe Timeout\r\n");
                    break;
                case DeviceState.TransactionCancelled:
                    System.Diagnostics.Debug.WriteLine("TransactionCancelled\r\n");
                    break;
                case DeviceState.DeviceTimeout:
                    SetOutputText("Device Timeout\r\n");
                    break;
                case DeviceState.TransactionFailed:                
                    SetOutputText("Transaction Failed: " + IDTechSDK.errorCode.getErrorString(transactionResultCode) + "\r\n");
                    break;
                case DeviceState.EMVCallback:
                    SetOutputText("EMV Callback\r\n");
                    break;
                case DeviceState.PINpadKeypress:
                    SetOutputText("PINPad Key was pressed\r\n");
                    break;
                default:
                    break;
            }
        }

        delegate void SetTextCallback(string text);

        private async void SetOutputText(string text)
        {
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
            {
                tbOutput.Text = text + "\r\n" + tbOutput.Text;
            });
        }

        private void SetOutputTextLog(string text)
        {
            logOutput.Text = text + "\r\n" + logOutput.Text;
        }

        private void btnGetFirmwareVersion_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() =>
            {
                string firmwareVersion = "";
                RETURN_CODE rt = IDT_L100.SharedController.device_getFirmwareVersion(ref firmwareVersion);
                if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
                {
                    SetOutputText("Firmware Ver: " + firmwareVersion + "\r\n");
                    System.Diagnostics.Debug.WriteLine("Firmware Ver: " + firmwareVersion);
                }
                else
                {
                    SetOutputText("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                    System.Diagnostics.Debug.WriteLine("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
                }
            });
        }

        private void btnGetPINEntry_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() =>
            {
                RETURN_CODE rt = IDT_L100.SharedController.pin_getEncryptedPIN(1, "4111111111111111", "ENTER PIN", 45);
                if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
                {
                    SetOutputText("Enter Pin Executed Successful\r\n");
                    System.Diagnostics.Debug.WriteLine("Enter Pin Executed Successful");
                }
                else
                {
                    SetOutputText("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                    System.Diagnostics.Debug.WriteLine("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
                }
            });
        }

        private void btnGetKeyInput_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() =>
            {
                RETURN_CODE rt = IDT_L100.SharedController.pin_promptForKeyInput(Convert.ToInt32(0), Convert.ToInt32(0), true, Convert.ToInt32(4), Convert.ToInt32(32), Convert.ToInt32(30));
                if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
                {
                    SetOutputText("Prompt for Key Entry Command Successful\r\n");
                    System.Diagnostics.Debug.WriteLine("Prompt for Key Entry Command Successful");
                }
                else
                {
                    SetOutputText("Prompt for Key Entry Command Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                    System.Diagnostics.Debug.WriteLine("Prompt for Key Entry Command Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
                }
            });
        }

        private void btnDisplayUnsecureMessage_Click(object sender, RoutedEventArgs e)
        {
            Task.Run(() =>
            {
                RETURN_CODE rt = IDT_Device.SharedController.lcd_displayMessage(1, "SmartPIN L100 Ready");
                if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
                {
                    SetOutputText("Display Message Successful\r\n");
                    System.Diagnostics.Debug.WriteLine("Display Message Successful");
                }
                else
                {
                    SetOutputText("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                    System.Diagnostics.Debug.WriteLine("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
                }
            });
        }
    }
}
