﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using IDTechSDK;
using System.Threading;

namespace L100_Demo
{
    public partial class Form1 : Form
    {
        //private BackgroundWorker backgroundWorker;
        delegate void SetTextCallback(string text);
        delegate void SetTextCallbackWithTextAlign(string text, bool withAlign = false);

        public Form1()
        {
            InitializeComponent();
            IDT_L100.setCallback(MessageCallBack);
            cbBaud.SelectedIndex = 0;
            buildL100Tree();
        }

        private void SetOutputText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (tbOutput.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetOutputText);
                Invoke(d, new object[] { text });
            }
            else
            {
                try { tbOutput.AppendText(text + "\r\n"); } catch (Exception ex) { }

            }
        }

        private void SetTitleText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTitleText);
                Invoke(d, new object[] { text });
            }
            else
            {
                this.Text = text;
            }
        }


        private void SetOutputTextLog(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (logOutput.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetOutputTextLog);
                Invoke(d, new object[] { text });
            }
            else
            {
                try { logOutput.AppendText(text + "\r\n"); }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine("Exception: " + ex);
                }

            }
        }

        private void SetConnectionText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (lblConnectionStatus.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetConnectionText);
                Invoke(d, new object[] { text });
            }
            else
            {
                lblConnectionStatus.Text = text;
            }
        }
        public static String GetTimestamp()
        {
            DateTime value = DateTime.Now;
            return value.ToString("HH:mm:ss.fff");
        }
        private void MessageCallBack(IDTechSDK.IDT_DEVICE_Types type, DeviceState state, byte[] data, IDTTransactionData cardData, EMV_Callback emvCallback, RETURN_CODE transactionResultCode)
        {
            if (emvCallback != null) System.Diagnostics.Debug.WriteLine("MessageCallBack: " + "LCD Display =" + Common.getHexStringFromBytes(emvCallback.lcd_messages));
            IDTechComm comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_USB);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_SERIAL);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_BT);
            if (comm == null) comm = Profile.getComm(type, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_AUDIO_JACK);
            DEVICE_INTERFACE_Types connect = DEVICE_INTERFACE_Types.DEVICE_INTERFACE_UNKNOWN;
            DEVICE_PROTOCOL_Types protocol = DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_UNKNOWN;
            if (comm != null)
            {
                connect = comm.getDeviceConnection();
                protocol = comm.getDeviceProtocol();
            }

            switch (state)
            {

                case DeviceState.ToConnect:
                    SetOutputText("To connect\n");
                    break;
                case DeviceState.DefaultDeviceTypeChange:
                    SetOutputText("\nSDK Default Device =  " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect));
                    SetTitleText("Universal SDK Demo:  " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect));
                    break;
                case DeviceState.Connected:
                    SetOutputText("\nConnected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect));
                    break;
                case DeviceState.Disconnected:
                    SetOutputText("\nDisconnected " + IDTechSDK.Profile.IDT_DEVICE_String(type, connect));
                    break;
                case DeviceState.ConnectionFailed:
                    SetOutputText("\nConnection Failed\n");
                    break;
                case DeviceState.Notification:
                    SetOutputText("\nNotification\n");
                    break;
                case DeviceState.TransactionData:
                    if (cardData == null) break;
                    //output parsed card data
                    SetOutputText("Return Code: " + transactionResultCode.ToString() + "\r\n");
                    if (cardData.Event == EVENT_TRANSACTION_DATA_Types.EVENT_TRANSACTION_PIN_DATA)
                    {
                        SetOutputText("PIN Data received:\r\nKSN: " + cardData.pin_KSN + "\r\nPINBLOCK: " + cardData.pin_pinblock + "\r\nKey Entry: " + cardData.pin_KeyEntry + "\r\n");
                        return;

                    }
                    break;
                case DeviceState.DataReceived:
                    SetOutputTextLog(GetTimestamp() + " IN: " + Common.getHexStringFromBytes(data));
                    break;
                case DeviceState.DataSent:
                    SetOutputTextLog(GetTimestamp() + " OUT: " + Common.getHexStringFromBytes(data));
                    break;
                case DeviceState.CommandTimeout:
                    SetOutputText("Command Timeout\n");
                    break;
                case DeviceState.ToSwipe:
                    SetOutputText("To Swipe\n");
                    break;
                case DeviceState.MSRDecodeError:
                    SetOutputText("MSR Decode Error\n");
                    break;
                case DeviceState.ToTap:
                    SetOutputText("To Tap\n");
                    break;
                case DeviceState.SwipeTimeout:
                    SetOutputText("Swipe Timeout\n");
                    break;
                case DeviceState.TransactionCancelled:
                    System.Diagnostics.Debug.WriteLine("TransactionCancelled.");
                    break;
                case DeviceState.DeviceTimeout:
                    SetOutputText("Device Timeout\n");
                    break;
                case DeviceState.TransactionFailed:

                    SetOutputText("Transaction Failed: " + IDTechSDK.errorCode.getErrorString(transactionResultCode) + "\r\n");
                    break;
                case DeviceState.EMVCallback:
                    SetOutputText("EMV Callback\n");
                    break;
                case DeviceState.PINpadKeypress:
                    SetOutputText("PINPad Key was pressed\n");
                    break;
                default:
                    SetConnectionText("Unknown");
                    break;

            }
        }


        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            IDT_L100.closeSerialPort();
            IDT_L100.closeUSB();

        }

        private void btnCloseSerial_Click(object sender, EventArgs e)
        {

            rbL100Serial.Checked = !(IDT_L100.closeSerialPort());
        }

        private void rbL100Serial_CheckedChanged(object sender, EventArgs e)
        {
            if (!rbL100Serial.Checked) return;
            bool isConnected = false;
            if (cbBaud.SelectedIndex == 0)
            {
                isConnected = IDT_L100.useSerialPort( Convert.ToInt32(serialPort.Text), false);
            }
            else
            {
                isConnected = IDT_L100.useSerialPort( Convert.ToInt32(serialPort.Text), Convert.ToInt32(cbBaud.SelectedItem), false);
            }
            if (!isConnected)
            {
                rbL100Serial.Checked = false;
                IDT_Device.setDeviceType(IDT_DEVICE_Types.IDT_DEVICE_NONE, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_UNKNOWN, DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_UNKNOWN);
                tbOutput.AppendText("Failed to locate device on specified COM port");
                return;
            }
            IDT_Device.setDeviceType(IDT_DEVICE_Types.IDT_DEVICE_L100, DEVICE_INTERFACE_Types.DEVICE_INTERFACE_SERIAL, DEVICE_PROTOCOL_Types.DEVICE_PROTOCOL_NGA);

        }

        private void rbL100_CheckedChanged(object sender, EventArgs e)
        {
            if (!rbL100.Checked) return;
            rbL100.Checked = IDT_L100.useUSB();

        }

        private void btnSDKVersion_Click(object sender, EventArgs e)
        {
            string ver = IDT_Device.SDK_Version();
            tbOutput.AppendText("SDK Version: " + ver + "\r\n");
            System.Diagnostics.Debug.WriteLine("SDK Version: " + ver);
            string[] vers = IDT_Device.getDLLVersions();
            tbOutput.AppendText("DLL Versions:\r\n");
            foreach (string str in vers)
            {
                tbOutput.AppendText(str);
            }
        }

        private void buildL100Tree()
        {
            TreeNode dnode1 = new TreeNode("Send Data Command");
            TreeNode fw = new TreeNode("Firmware Version");
            TreeNode rd = new TreeNode("Reboot Device");
            TreeNode gks = new TreeNode("Get Key Status");
            TreeNode sdt = new TreeNode("Set Date/Time");
            TreeNode gdt = new TreeNode("Get Date/Time");
            TreeNode esm = new TreeNode("Enter Stop Mode");
            TreeNode ssm = new TreeNode("Set Sleep Mode");
            TreeNode rki = new TreeNode("Execute RKI");

            TreeNode[] darray = new TreeNode[] { dnode1, fw, rd, gks, sdt, gdt, esm, ssm, rki };
            TreeNode device = new TreeNode("Device", darray);
            tvCommands.Nodes.Add(device);

            TreeNode pnode1 = new TreeNode("Get Encrypted PIN");
            TreeNode pke = new TreeNode("Prompt Key Entry");
            TreeNode pae = new TreeNode("Prompt Amt Entry");
            TreeNode ce = new TreeNode("Cancel PIN Entry");
            TreeNode getf = new TreeNode("Get Function Key");
            TreeNode beep = new TreeNode("Send Beep");



            TreeNode[] parray = new TreeNode[] { pnode1, pke, pae, getf, ce, beep };
            TreeNode pin = new TreeNode("Pin", parray);
            tvCommands.Nodes.Add(pin);

            TreeNode sn = new TreeNode("Get Serial Number");
            TreeNode mn = new TreeNode("Get Model Number");
            TreeNode sbr = new TreeNode("Set Baud");
            TreeNode gbr = new TreeNode("Get Baud");
            TreeNode[] carray = new TreeNode[] { sn, mn, sbr, gbr };
            TreeNode config = new TreeNode("Config", carray);
            tvCommands.Nodes.Add(config);

            TreeNode ctd = new TreeNode("Clear Display");
            TreeNode cln = new TreeNode("Clear Line #");
            TreeNode stp = new TreeNode("Save Prompt");
            TreeNode dtp = new TreeNode("Display Prompt");
            TreeNode dtm = new TreeNode("Display Message");
            TreeNode etb = new TreeNode("Enable Backlight");
            TreeNode disb = new TreeNode("Disable Backlight");
            TreeNode tbs = new TreeNode("Backlight Status");
            TreeNode[] lcdarray = new TreeNode[] {ctd, cln, stp, dtp, dtm, etb, disb, tbs };
            TreeNode lcd = new TreeNode("LCD", lcdarray);
            tvCommands.Nodes.Add(lcd);

        }
        private string command = "";
        private void button1_Click(object sender, EventArgs e)
        {

           // if (tvCommands.SelectedNode == null) return;
          //  string command = tvCommands.SelectedNode.Text;
            if (command.Equals("Reboot Device")) btnReboot_Click(null, null);
            if (command.Equals("Send Data Command")) btnSendDataCommand_Click(null, null);
            if (command.Equals("Firmware Version")) btnFirmware_Click(null, null);
            if (command.Equals("Get Encrypted PIN")) btnEncryptedPIN_Click(null, null);
            if (command.Equals("Get Serial Number")) btnGetSerialNumber_Click(null, null);
            if (command.Equals("Get Model Number")) btnModelNumber_Click(null, null);
            if (command.Equals("Get Key Status")) btnGetKeyStatus_Click(null, null);
            if (command.Equals("Set Date/Time")) btnsetDateTime_Click(null, null);
            if (command.Equals("Get Date/Time")) btngetDateTime_Click(null, null);
            if (command.Equals("Enter Stop Mode")) btnenterSleepMode_Click(null, null);
            if (command.Equals("Set Sleep Mode")) btnsetSleepMode_Click(null, null);
            if (command.Equals("Prompt Key Entry")) btnpinKeyEntry_Click(null, null);
            if (command.Equals("Prompt Amt Entry")) btnpinAmountEntry_Click(null, null);
            if (command.Equals("Cancel PIN Entry")) btnCancelPIN_Click(null, null);
            if (command.Equals("Get Function Key")) btngetFunctionKey_Click(null, null);
            if (command.Equals("Send Beep")) btnbeep_Click(null, null);
            if (command.Equals("Clear Line #")) btnclearLineNumber_Click(null, null);
            if (command.Equals("Save Prompt")) btnsavePrompt_Click(null, null);
            if (command.Equals("Display Prompt")) btndisplayPrompt_Click(null, null);
            if (command.Equals("Display Message")) btndisplayMessage_Click(null, null);
            if (command.Equals("Enable Backlight")) btnenableBacklight_Click(null, null);
            if (command.Equals("Disable Backlight")) btndisableBacklight_Click(null, null);
            if (command.Equals("Backlight Status")) btnbacklightStatus_Click(null, null);
            if (command.Equals("Set Baud")) btnSetBaud_Click(null, null);
            if (command.Equals("Get Baud")) btnGetBaud_Click(null, null);
            if (command.Equals("Clear Display")) btnclearDisplay_Click(null, null);
            if (command.Equals("Execute RKI")) btnStartRKI_Click(null, null);


        }

        private void clearPanels()
        {
            panelSleepMode.Visible = false;
            panelPINEntry.Visible = false;
            panelBeep.Visible = false;
            panelLCD.Visible = false;
            panelBaud.Visible = false;
            ngaPanel.Visible = false;
        }




        private void tvCommands_AfterSelect(object sender, TreeViewEventArgs e)
        {
            command = e.Node.Text;
            clearPanels();
            if (command.Equals("Send Data Command")) ngaPanel.Visible = true;
            if (command.Equals("Set Sleep Mode")) panelSleepMode.Visible = true;
            if (command.Equals("Prompt Key Entry")) panelPINEntry.Visible = true;
            if (command.Equals("Prompt Amt Entry")) panelPINEntry.Visible = true;
            if (command.Equals("Send Beep")) panelBeep.Visible = true;
            if (command.Equals("Clear Line #")) panelLCD.Visible = true;
            if (command.Equals("Save Prompt")) panelLCD.Visible = true;
            if (command.Equals("Display Prompt")) panelLCD.Visible = true;
            if (command.Equals("Display Message")) panelLCD.Visible = true;
            if (command.Equals("Set Baud")) panelBaud.Visible = true;

        }
        private void btnGetBaud_Click(object sender, EventArgs e)
        {
            int index = 0;
            RETURN_CODE rt = IDT_L100.SharedController.config_getBaudRate(ref index);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                switch (index)
                {
                    case 2:
                        tbOutput.AppendText("Baud Rate 2400\r\n");
                        break;

                    case 3:
                        tbOutput.AppendText("Baud Rate 4800\r\n");
                        break;

                    case 4:
                        tbOutput.AppendText("Baud Rate 9600\r\n");
                        break;

                    case 6:
                        tbOutput.AppendText("Baud Rate 19200\r\n");
                        break;

                    case 7:
                        tbOutput.AppendText("Baud Rate 38400\r\n");
                        break;

                    case 9:
                        tbOutput.AppendText("Baud Rate 115200\r\n");
                        break;

                }
            }
            else
            {
                tbOutput.AppendText("Get Baud failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Baud failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnclearDisplay_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_Device.SharedController.lcd_clearAllLines();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Clear Display Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Clear Display Successful");
            }
            else
            {
                tbOutput.AppendText("Clear Line Number failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Clear Line Number failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btnStartRKI_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_Device.SharedController.device_startRKI();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("\r\nRKI Successful");
                System.Diagnostics.Debug.WriteLine("nRKI Successful ");
            }
            else
            {
                tbOutput.AppendText("RKI Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("RKI Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnSetBaud_Click(object sender, EventArgs e)
        {
            int index = cbsetBaud.SelectedIndex;
            if (index == -1 || index == 0 || index == 4 || index == 7)
            {
                tbOutput.AppendText("Invalid Baud Rate Selected\r\n");
                return;
            }
            RETURN_CODE rt = IDT_L100.SharedController.config_setBaudRate(index + 1);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Set Baud Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Set Baud Successful");
            }
            else
            {
                tbOutput.AppendText("Set Baud failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Set Baud failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnbacklightStatus_Click(object sender, EventArgs e)
        {
            bool status = false;
            RETURN_CODE rt = IDT_L100.SharedController.lcd_getBacklightStatus(ref status);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Backlight status: " + (status ? "ENABLED" : "DISABLED") + "\r\n");
                System.Diagnostics.Debug.WriteLine("Backlight status: " + (status ? "ENABLED" : "DISABLED"));
            }
            else
            {
                tbOutput.AppendText("Get backlight status failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get backlight status failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btndisableBacklight_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.lcd_enableBacklight(false);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Backlight Disabled Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Backlight Disabled Successful");
            }
            else
            {
                tbOutput.AppendText("Backlight Disabled failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Backlight Disabled failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnenableBacklight_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.lcd_enableBacklight(true);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Backlight Enabled Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Backlight Enabled Successful");
            }
            else
            {
                tbOutput.AppendText("Backlight Enabled failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Backlight Enabled failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btndisplayMessage_Click(object sender, EventArgs e)
        {
            if (tblcdLine.Text.Length == 0)
            {
                tbOutput.AppendText("Missing Required Parameters\r\n");
                return;
            }
            RETURN_CODE rt = IDT_L100.SharedController.lcd_displayMessage(Convert.ToInt32(tblcdLine.Text), tbLCDMessage.Text);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Display Message Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Display Message Successful");
            }
            else
            {
                tbOutput.AppendText("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Display Message failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }
        private void btndisplayPrompt_Click(object sender, EventArgs e)
        {
            if (tbLCDPrompt.Text.Length == 0 || tblcdLine.Text.Length == 0)
            {
                tbOutput.AppendText("Missing Required Parameters\r\n");
                return;
            }
            RETURN_CODE rt = IDT_L100.SharedController.lcd_displayPrompt(Convert.ToInt32(tbLCDPrompt.Text), Convert.ToInt32(tblcdLine.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Save Prompt Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Save Prompt Successful");
            }
            else
            {
                tbOutput.AppendText("Save Prompt failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Save Prompt failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btnsavePrompt_Click(object sender, EventArgs e)
        {
            if (tbLCDPrompt.Text.Length == 0)
            {
                tbOutput.AppendText("Missing Required Parameters\r\n");
                return;
            }
            RETURN_CODE rt = IDT_L100.SharedController.lcd_savePrompt(Convert.ToInt32(tbLCDPrompt.Text), tbLCDMessage.Text);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Save Prompt Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Save Prompt Successful");
            }
            else
            {
                tbOutput.AppendText("Save Prompt failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Save Prompt failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btnclearLineNumber_Click(object sender, EventArgs e)
        {
            if (tblcdLine.Text.Length == 0)
            {
                tbOutput.AppendText("Missing Required Parameters\r\n");
                return;
            }
            
            RETURN_CODE rt = IDT_L100.SharedController.lcd_clearDisplay(Convert.ToInt32(tblcdLine.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Clear Line Number Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Clear Line Number Successful");
            }
            else
            {
                tbOutput.AppendText("Clear Line Number failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Clear Line Number failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btnbeep_Click(object sender, EventArgs e)
        {
            if (tbFrequency.Text.Length == 0 || tbDuration.Text.Length == 0)
            {
                tbOutput.AppendText("Missing Required Parameters\r\n");
                return;
            }

            RETURN_CODE rt = IDT_L100.SharedController.pin_sendBeep(Convert.ToInt32(tbFrequency.Text), Convert.ToInt32(tbDuration.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Beep  successful\r\n");
                System.Diagnostics.Debug.WriteLine("Beep  successful");
            }
            else
            {
                tbOutput.AppendText("Beep failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Beep failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btngetFunctionKey_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_getFunctionKey(30);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Get Function Key  successful\r\n");
                System.Diagnostics.Debug.WriteLine("Get Function Key  successful");
            }
            else
            {
                tbOutput.AppendText("Get Function Key failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Function Key failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnCancelPIN_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_cancelPINEntry();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Cancel PIN Entry  successful\r\n");
                System.Diagnostics.Debug.WriteLine("Cancel PIN Entry  successful");
            }
            else
            {
                tbOutput.AppendText("Cancel PIN Entry failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Cancel PIN Entry failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }
        private void btnpinAmountEntry_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_promptForAmountInput(Convert.ToInt32(tbMessageID.Text), Convert.ToInt32(tbLanguageID.Text), Convert.ToInt32(tbMinLength.Text), Convert.ToInt32(tbMaxLength.Text), Convert.ToInt32(tbPinTimout.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Prompt for Key Entry Command Successful\r\n");
            }
            else
            {
                tbOutput.AppendText("Prompt for Key Entry Command Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
            }
        }
        private void btnpinKeyEntry_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_promptForKeyInput(Convert.ToInt32(tbMessageID.Text), Convert.ToInt32(tbLanguageID.Text), cbMaskInput.Checked, Convert.ToInt32(tbMinLength.Text), Convert.ToInt32(tbMaxLength.Text), Convert.ToInt32(tbPinTimout.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Prompt for Key Entry Command Successful\r\n");
            }
            else
            {
                tbOutput.AppendText("Prompt for Key Entry Command Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
            }
        }

        private void btnsetSleepMode_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.device_setSleepModeTime(Convert.ToInt32(tbSleepMode.Text));
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Set Sleep Mode  successful\r\n");
                System.Diagnostics.Debug.WriteLine("Set Sleep Mode  successful");
            }
            else
            {
                tbOutput.AppendText("Set Sleep Mode failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Set Sleep Mode failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }
        private void btnenterSleepMode_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.device_enterStopMode();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Enter Stop Mode  successful\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Stop Mode  successful");
            }
            else
            {
                tbOutput.AppendText("Enter Stop Mode failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Stop Mode failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnsetDateTime_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.device_setDateTime();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {

                tbOutput.AppendText("Set Date Time Successful" + "\r\n");
                System.Diagnostics.Debug.WriteLine("Set Date Time Successful");
            }
            else
            {
                tbOutput.AppendText("Set Date Time Unsuccessful" + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Set Date Time Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btngetDateTime_Click(object sender, EventArgs e)
        {
            byte[] dt = null;
            RETURN_CODE rt = IDT_L100.SharedController.device_getDateTime(ref dt);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS && dt != null)
            {

                tbOutput.AppendText("Get Date Time Successful " + Common.getHexStringFromBytes(dt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Date Time Successful");
            }
            else
            {
                tbOutput.AppendText("Get Date Time Unsuccessful" + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Date Time Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }

        }

        private void btnGetKeyStatus_Click(object sender, EventArgs e)
        {
            byte[] status = null;
            RETURN_CODE rt = IDT_L100.SharedController.device_getKeyStatus(ref status);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS && status != null)
            {
                tbOutput.AppendText("Get Key Status Error successful\r\n");
                if (status[0] == 0x01)
                    tbOutput.AppendText("PIN DUKPT Key Loaded\r\n");
                else if (status[0] == 0xFF)
                    tbOutput.AppendText("PIN DUKPT Key STOP\r\n");
                else
                    tbOutput.AppendText("PIN DUKPT Key NOT Loaded\r\n");
                if (status[1] == 0x01)
                    tbOutput.AppendText("PIN Master Key Loaded\r\n");
                else
                    tbOutput.AppendText("PIN Master Key NOT Loaded\r\n");
                if (status[2] == 0x01)
                    tbOutput.AppendText("PIN Session Key Loaded\r\n");
                else
                    tbOutput.AppendText("PIN Session Key NOT Loaded\r\n");
                if (status[3] == 0x01)
                    tbOutput.AppendText("MSR DUKPT Key Loaded\r\n");
                else if (status[3] == 0xFF)
                    tbOutput.AppendText("MSR DUKPT Key STOP\r\n");
                else
                    tbOutput.AppendText("MSR DUKPT Key NOT Loaded\r\n");
                if (status[4] == 0x01)
                    tbOutput.AppendText("ICC DUKPT Key Loaded\r\n");
                else if (status[4] == 0xFF)
                    tbOutput.AppendText("ICC DUKPT Key STOP\r\n");
                else
                    tbOutput.AppendText("ICC DUKPT Key NOT Loaded\r\n");
                if (status[5] == 0x01)
                    tbOutput.AppendText("Admin Key Loaded\r\n");
                else if (status[5] == 0xFF)
                    tbOutput.AppendText("Admin Key STOP\r\n");
                else
                    tbOutput.AppendText("Admin Key NOT Loaded\r\n");

                System.Diagnostics.Debug.WriteLine("USB Reboot successfully");
            }
            else
            {
                tbOutput.AppendText("Get Key Status Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Key Status Error failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnModelNumber_Click(object sender, EventArgs e)
        {
            string modelNumber = "";

            RETURN_CODE rt = IDT_L100.SharedController.config_getModelNumber(ref modelNumber);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Model Number: " + modelNumber + "\r\n");
                System.Diagnostics.Debug.WriteLine("Model Number: " + modelNumber);
            }
            else
            {
                tbOutput.AppendText("Get Model Number Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Model Number Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnGetSerialNumber_Click(object sender, EventArgs e)
        {
            string sNumber = "";

            RETURN_CODE rt = IDT_L100.SharedController.config_getSerialNumber(ref sNumber);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Serial Number: " + sNumber + "\r\n");
                System.Diagnostics.Debug.WriteLine("Serial Number: " + sNumber);
            }
            else
            {
                tbOutput.AppendText("Get Serial Number Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Serial Number Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnEncryptedPIN_Click(object sender, EventArgs e)
        {
            RETURN_CODE rt = IDT_L100.SharedController.pin_getEncryptedPIN(1, "4111111111111111", "ENTER PIN", 45);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Enter Pin Executed Successful\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Pin Executed Successful");
            }
            else
            {
                tbOutput.AppendText("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Enter Pin Executed failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnReboot_Click(object sender, EventArgs e)
        {

            RETURN_CODE rt = IDT_L100.SharedController.device_rebootDevice();
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("USB Reboot successfully\r\n");
                System.Diagnostics.Debug.WriteLine("USB Reboot successfully");
            }
            else
            {
                tbOutput.AppendText("USB Reboot failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("USB Reboot failed Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnSendDataCommand_Click(object sender, EventArgs e)
        {
            if (tbCommand.Text.Length == 0) return;
            String command = tbCommand.Text;
            command = command.Replace(" ", "");
            byte[] response = new byte[0];
            RETURN_CODE rt = IDT_L100.SharedController.device_sendDataCommand(tbCommand.Text, cbLRC.Checked, ref response);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {

                tbOutput.AppendText("Send Command Response HEX: " + Common.getHexStringFromBytes(response) + "\r\n");

                for (int x = 0; x < response.Length; x++)
                {
                    if (response[x] < 0x20 || response[x] > 0x7d) response[x] = 0x3f;

                }
                tbOutput.AppendText("Send Command Response ASCII: " + System.Text.Encoding.UTF8.GetString(response) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Send Command Response: " + Common.getHexStringFromBytes(response));
            }
            else
            {
                tbOutput.AppendText("Send Command Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Send Command Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnFirmware_Click(object sender, EventArgs e)
        {
            string firmwareVersion = "";
            RETURN_CODE rt = IDT_L100.SharedController.device_getFirmwareVersion(ref firmwareVersion);
            if (rt == RETURN_CODE.RETURN_CODE_DO_SUCCESS)
            {
                tbOutput.AppendText("Firmware Ver: " + firmwareVersion + "\r\n");
                System.Diagnostics.Debug.WriteLine("Firmware Ver: " + firmwareVersion);
            }
            else
            {
                tbOutput.AppendText("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt) + ": " + IDTechSDK.errorCode.getErrorString(rt) + "\r\n");
                System.Diagnostics.Debug.WriteLine("Get Firmware Fail Error Code: " + "0x" + String.Format("{0:X}", (ushort)rt));
            }
        }

        private void btnClearLogs_Click(object sender, EventArgs e)
        {
            logOutput.Text = "";
            tbOutput.Text = "";
        }
    }
}
